@@ -106,8 +106,8 @@ const unsigned int OSC_DAC_BITS = 12;
  * On my 6581R4AR has 0x3A as the only value giving the same output level as 1.prg
  */
 //@{
-unsigned int constexpr OFFSET_6581 = 0x380;
-unsigned int constexpr OFFSET_8580 = 0x9c0;
+const unsigned int OFFSET_6581 = 0x380;
+const unsigned int OFFSET_8580 = 0x9c0;
 //@}
 
 /**
@@ -128,8 +128,8 @@ unsigned int constexpr OFFSET_8580 = 0x9c0;
  * [2]: http://noname.c64.org/csdb/forums/?roomid=11&topicid=29025&showallposts=1
  */
 //@{
-int constexpr BUS_TTL_6581 = 0x01d00;
-int constexpr BUS_TTL_8580 = 0xa2000;
+const int BUS_TTL_6581 = 0x01d00;
+const int BUS_TTL_8580 = 0xa2000;
 //@}
 
 SID::SID() :
@@ -211,11 +211,13 @@ void SID::setChipModel(ChipModel model)
     {
     case MOS6581:
         filter = filter6581.get();
+        scaleFactor = 3;
         modelTTL = BUS_TTL_6581;
         break;
 
     case MOS8580:
         filter = filter8580.get();
+        scaleFactor = 5;
         modelTTL = BUS_TTL_8580;
         break;
 
@@ -226,7 +228,8 @@ void SID::setChipModel(ChipModel model)
     this->model = model;
 
     // calculate waveform-related tables
-    matrix_t* tables = WaveformCalculator::getInstance()->buildTable(model);
+    matrix_t* wavetables = WaveformCalculator::getInstance()->getWaveTable();
+    matrix_t* pulldowntables = WaveformCalculator::getInstance()->buildPulldownTable(model);
 
     // calculate envelope DAC table
     {
@@ -261,7 +264,8 @@ void SID::setChipModel(ChipModel model)
         voice[i]->setEnvDAC(envDAC);
         voice[i]->setWavDAC(oscDAC);
         voice[i]->wave()->setModel(is6581);
-        voice[i]->wave()->setWaveformModels(tables);
+        voice[i]->wave()->setWaveformModels(wavetables);
+        voice[i]->wave()->setPulldownModels(pulldowntables);
     }
 }
 
